package com.thelastdaytosurvive.game;
import org.usb4java.Device;
import com.badlogic.gdx.utils.TimeUtils;
import com.thelastdaytosurvive.game.McuWithPeriBoard;

public class PracticumController
{
	int count = 0;
	long lastLEDTime = TimeUtils.millis();
	long refreshRate = 500;
	McuWithPeriBoard peri;
	
	private static final int AXIS_MID = 500;
	private static final int AXIS_MID_ERROR = 10;
	private static final int AXIS_MAX = 1023;
	
	public PracticumController(){
		System.out.println("Test");
		McuBoard.initUsb();
		try
        {
        	Device[] devices = McuBoard.findBoards();
        	
        	if (devices.length == 0) {
                System.out.format("** Practicum board not found **\n");
                return;
        	}
        	else {
                System.out.format("** Found %d practicum board(s) **\n", devices.length);
        	}
            peri = new McuWithPeriBoard(devices[0]);
            System.out.format("** Practicum board found **\n");
            System.out.format("** Manufacturer: %s\n", peri.getManufacturer());
            System.out.format("** Product: %s\n", peri.getProduct());          
        }
        catch (Exception e)
        {
            System.out.println(e);
        }
	}
	
	public void update(){
		if(TimeUtils.millis() - lastLEDTime > refreshRate)
        {
             boolean joy1Btn = getJoy1Btn();
             float joy1X = getJoy1X();
             float joy1Y = getJoy1Y();
             
             System.out.format("Joy 1 -> btn : %s | X : %f | Y : %f |||",
             		joy1Btn, joy1X, joy1Y);
             
             boolean joy2Btn = getJoy2Btn();
             float joy2X = getJoy2X();
             float joy2Y = getJoy2Y();
             System.out.format("Joy 2 -> btn : %s | X : %f | Y : %f\n",
             		joy2Btn, joy2X, joy2Y);
        }
		
	}
	
	public float getJoy1X(){
		int joy1Y = peri.getJoy1Y();
		if(joy1Y >= (AXIS_MID - AXIS_MID_ERROR) && joy1Y <= (AXIS_MID + AXIS_MID_ERROR)){
			return 0f;
		}
		else{
			float ans = ((float)joy1Y - (float)AXIS_MID) / ((float)AXIS_MID);
			if(ans < -1f) ans = -1f;
			if(ans > 1f) ans = 1f;
			return ans;
		}
	}
	
	public float getJoy1Y(){
		int joy1X = peri.getJoy1X();
		if(joy1X >= (AXIS_MID - AXIS_MID_ERROR) && joy1X <= (AXIS_MID + AXIS_MID_ERROR)){
			return 0f;
		}
		else{
			float ans = -((float)joy1X - (float)AXIS_MID) / ((float)AXIS_MID);
			if(ans < -1f) ans = -1f;
			if(ans > 1f) ans = 1f;
			return ans;
		}
	}
	
	public boolean getJoy1Btn(){
		return peri.getJoy1Btn();
	}
	
	public float getJoy2X(){
		int joy2Y = peri.getJoy2Y();
		if(joy2Y >= (AXIS_MID - AXIS_MID_ERROR) && joy2Y <= (AXIS_MID + AXIS_MID_ERROR)){
			return 0f;
		}
		else{
			float ans = ((float)joy2Y - (float)AXIS_MID) / ((float)AXIS_MID);
			if(ans < -1f) ans = -1f;
			if(ans > 1f) ans = 1f;
			return ans;
		}
	}
	
	public float getJoy2Y(){
		int joy2X = peri.getJoy2X();
		if(joy2X >= (AXIS_MID - AXIS_MID_ERROR) && joy2X <= (AXIS_MID + AXIS_MID_ERROR)){
			return 0f;
		}
		else{
			float ans = -((float)joy2X - (float)AXIS_MID) / ((float)AXIS_MID);
			if(ans < -1f) ans = -1f;
			if(ans > 1f) ans = 1f;
			return ans;
		}
	}
	
	public boolean getJoy2Btn(){
		return peri.getJoy2Btn();
	}
	
	public boolean getFireBtn(){
		return peri.getFireBtn();
	}
	
	public boolean getReloadBtn(){
		return peri.getReloadBtn();
	}
	
	public void setLed(int value){
		peri.setLed(value);
	}
}
