#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "peri.h"
#include "usbdrv.h"

#define RQ_GET_JOY1_X		0
#define RQ_GET_JOY1_Y		1
#define RQ_GET_JOY1_BTN		2
#define RQ_GET_JOY2_X		3
#define RQ_GET_JOY2_Y		4
#define RQ_GET_JOY2_BTN		5
#define RQ_GET_FIRE_BTN		6
#define RQ_GET_RELOAD_BTN	7
#define RQ_SET_LED			8


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint8_t switch_state;
    static uint16_t joy_stick_value;

	if(rq->bRequest == RQ_GET_JOY1_X)
	{
		joy_stick_value = read_adc(0);
		usbMsgPtr = &joy_stick_value;
	
		return sizeof(uint16_t);
	}
	else if(rq->bRequest == RQ_GET_JOY1_Y)
	{
		joy_stick_value = read_adc(2);
		usbMsgPtr = &joy_stick_value;
	
		return sizeof(uint16_t);
	}
	else if(rq->bRequest == RQ_GET_JOY1_BTN)
	{
		switch_state = IS_C_SWITCH_PRESSED(4);

        usbMsgPtr = &switch_state;

        return 1;
	}
	else if(rq->bRequest == RQ_GET_JOY2_X)
	{
		joy_stick_value = read_adc(1);
		usbMsgPtr = &joy_stick_value;
	
		return sizeof(uint16_t);
	}
	else if(rq->bRequest == RQ_GET_JOY2_Y)
	{
		joy_stick_value = read_adc(3);
		usbMsgPtr = &joy_stick_value;
	
		return sizeof(uint16_t);
	}
	else if(rq->bRequest == RQ_GET_JOY2_BTN)
	{
		switch_state = IS_C_SWITCH_PRESSED(5);

        usbMsgPtr = &switch_state;

        return 1;
	}
	else if(rq->bRequest == RQ_GET_FIRE_BTN)
	{
		switch_state = IS_D_SWITCH_PRESSED(0);

		usbMsgPtr = &switch_state;

        return 1;
	}
	else if(rq->bRequest == RQ_GET_RELOAD_BTN)
	{
		switch_state = IS_D_SWITCH_PRESSED(1);

		usbMsgPtr = &switch_state;

        return 1;
	}
	else if(rq->bRequest == RQ_SET_LED)
	{
		uint8_t led_val = rq->wValue.bytes[0];
		if(led_val == 5)
		{
			set_led(4,1);
			set_led(3,1);
			set_led(2,1);
			set_led(1,1);
			set_led(0,1);
		}
		else if(led_val == 4)
		{
			set_led(4,0);
			set_led(3,1);
			set_led(2,1);
			set_led(1,1);
			set_led(0,1);
		}
		else if(led_val == 3)
		{
			set_led(4,0);
			set_led(3,0);
			set_led(2,1);
			set_led(1,1);
			set_led(0,1);
		}
		else if(led_val == 2)
		{
			set_led(4,0);
			set_led(3,0);
			set_led(2,0);
			set_led(1,1);
			set_led(0,1);
		}
		else if(led_val == 1)
		{
			set_led(4,0);
			set_led(3,0);
			set_led(2,0);
			set_led(1,0);
			set_led(0,1);
		}
		else
		{
			set_led(4,0);
			set_led(3,0);
			set_led(2,0);
			set_led(1,0);
			set_led(0,0);
		}
	}
    return 0;
}

/* ------------------------------------------------------------------------- */
int main(void)
{
    init_peripheral();

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
